r"""Contain plotting functionalities."""

from __future__ import annotations

__all__ = ["binary_precision_recall_curve", "binary_roc_curve"]

from arkas.plot.pr import binary_precision_recall_curve
from arkas.plot.roc import binary_roc_curve
