r"""Contain sections."""

from __future__ import annotations

__all__ = [
    "AccuracySection",
    "BaseSection",
    "BinaryPrecisionSection",
    "ContentSection",
    "ResultSection",
]

from arkas.section.accuracy import AccuracySection
from arkas.section.base import BaseSection
from arkas.section.binary_precision import BinaryPrecisionSection
from arkas.section.content import ContentSection
from arkas.section.result import ResultSection
